/* Copyright 1998, 2005 The Apache Software Foundation or its licensors, as applicable
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package tests.api.java.io;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.SequenceInputStream;
import java.util.Enumeration;

public class SequenceInputStreamTest extends junit.framework.TestCase {

	SequenceInputStream si;

	String s1 = "Hello";

	String s2 = "World";

	/**
	 * @tests java.io.SequenceInputStream#SequenceInputStream(java.io.InputStream,
	 *        java.io.InputStream)
	 */
	public void test_ConstructorLjava_io_InputStreamLjava_io_InputStream() {
		// Test for method java.io.SequenceInputStream(java.io.InputStream,
		// java.io.InputStream)
		// Used in tests
	}

	/**
	 * @tests java.io.SequenceInputStream#SequenceInputStream(java.util.Enumeration)
	 */
	public void test_ConstructorLjava_util_Enumeration() {
		// Test for method java.io.SequenceInputStream(java.util.Enumeration)
		class StreamEnumerator implements Enumeration {
			InputStream streams[] = new InputStream[2];

			int count = 0;

			public StreamEnumerator() {
				streams[0] = new ByteArrayInputStream(s1.getBytes());
				streams[1] = new ByteArrayInputStream(s2.getBytes());
			}

			public boolean hasMoreElements() {
				return count < streams.length;
			}

			public Object nextElement() {
				return streams[count++];
			}
		}

		try {
			si = new SequenceInputStream(new StreamEnumerator());
			byte buf[] = new byte[s1.length() + s2.length()];
			si.read(buf, 0, s1.length());
			si.read(buf, s1.length(), s2.length());
			assertTrue("Read incorrect bytes: " + new String(buf), new String(
					buf).equals(s1 + s2));
		} catch (IOException e) {
			fail("IOException during read test : " + e.getMessage());
		}

	}

	/**
	 * @tests java.io.SequenceInputStream#available()
	 */
	public void test_available() {
		// Test for method int java.io.SequenceInputStream.available()
		try {

			assertTrue("Returned incorrect number of bytes: " + si.available(),
					si.available() == s1.length());
		} catch (IOException e) {
			fail("IOException during available test : " + e.getMessage());
		}
	}

	/**
	 * @tests java.io.SequenceInputStream#close()
	 */
	public void test_close() {
		// Test for method void java.io.SequenceInputStream.close()
		try {
			si.close();
		} catch (IOException e) {
			fail("IOException during close test : " + e.getMessage());
		}

	}

	/**
	 * @tests java.io.SequenceInputStream#read()
	 */
	public void test_read() {
		// Test for method int java.io.SequenceInputStream.read()
		try {
			si.read();
			assertTrue("Read incorrect char", (char) si.read() == s1.charAt(1));
		} catch (IOException e) {
			fail("IOException during read test: " + e.getMessage());
		}
	}

	/**
	 * @tests java.io.SequenceInputStream#read(byte[], int, int)
	 */
	public void test_read$BII() {
		// Test for method int java.io.SequenceInputStream.read(byte [], int,
		// int)
		try {
			byte buf[] = new byte[s1.length() + s2.length()];
			si.read(buf, 0, s1.length());
			si.read(buf, s1.length(), s2.length());
			assertTrue("Read incorrect bytes: " + new String(buf), new String(
					buf).equals(s1 + s2));
		} catch (IOException e) {
			fail("IOException during read test : " + e.getMessage());
		}
	}

	/**
	 * Sets up the fixture, for example, open a network connection. This method
	 * is called before a test is executed.
	 */
	protected void setUp() {
		si = new SequenceInputStream(new ByteArrayInputStream(s1.getBytes()),
				new ByteArrayInputStream(s2.getBytes()));
	}

	/**
	 * Tears down the fixture, for example, close a network connection. This
	 * method is called after a test is executed.
	 */
	protected void tearDown() {
	}
}
