/* Copyright 1998, 2005 The Apache Software Foundation or its licensors, as applicable
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package tests.api.java.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import tests.support.Support_ListTest;

public class ArrayListTest extends junit.framework.TestCase {

	List alist;

	static Object[] objArray;
	{
		objArray = new Object[100];
		for (int i = 0; i < objArray.length; i++)
			objArray[i] = new Integer(i);
	}

	/**
	 * @tests java.util.ArrayList#ArrayList()
	 */
	public void test_Constructor() {
		// Test for method java.util.ArrayList()
		new Support_ListTest("", alist).runTest();

		ArrayList subList = new ArrayList();
		for (int i = -50; i < 150; i++)
			subList.add(new Integer(i));
		new Support_ListTest("", subList.subList(50, 150)).runTest();
	}

	/**
	 * @tests java.util.ArrayList#ArrayList(int)
	 */
	public void test_ConstructorI() {
		// Test for method java.util.ArrayList(int)
		ArrayList al = new ArrayList(5);
		assertTrue("Incorrect arrayList created", al.size() == 0);
	}

	/**
	 * @tests java.util.ArrayList#ArrayList(java.util.Collection)
	 */
	public void test_ConstructorLjava_util_Collection() {
		// Test for method java.util.ArrayList(java.util.Collection)
		ArrayList al = new ArrayList(Arrays.asList(objArray));
		assertTrue("arrayList created from collection has incorrect size", al
				.size() == objArray.length);
		for (int counter = 0; counter < objArray.length; counter++)
			assertTrue(
					"arrayList created from collection has incorrect elements",
					al.get(counter) == objArray[counter]);

	}

	/**
	 * @tests java.util.ArrayList#add(int, java.lang.Object)
	 */
	public void test_addILjava_lang_Object() {
		// Test for method void java.util.ArrayList.add(int, java.lang.Object)
		Object o;
		alist.add(50, o = new Object());
		assertTrue("Failed to add Object", alist.get(50) == o);
		assertTrue("Failed to fix up list after insert",
				alist.get(51) == objArray[50]
						&& (alist.get(52) == objArray[51]));
		Object oldItem = alist.get(25);
		alist.add(25, null);
		assertTrue("Should have returned null", alist.get(25) == null);
		assertTrue("Should have returned the old item from slot 25", alist
				.get(26) == oldItem);
	}

	/**
	 * @tests java.util.ArrayList#add(java.lang.Object)
	 */
	public void test_addLjava_lang_Object() {
		// Test for method boolean java.util.ArrayList.add(java.lang.Object)
		Object o = new Object();
		alist.add(o);
		assertTrue("Failed to add Object", alist.get(alist.size() - 1) == o);
		alist.add(null);
		assertTrue("Failed to add null", alist.get(alist.size() - 1) == null);
	}

	/**
	 * @tests java.util.ArrayList#addAll(int, java.util.Collection)
	 */
	public void test_addAllILjava_util_Collection() {
		// Test for method boolean java.util.ArrayList.addAll(int,
		// java.util.Collection)
		alist.addAll(50, alist);
		assertTrue("Returned incorrect size after adding to existing list",
				alist.size() == 200);
		for (int i = 0; i < 50; i++)
			assertTrue("Manipulated elements < index",
					alist.get(i) == objArray[i]);
		for (int i = 0; i >= 50 && (i < 150); i++)
			assertTrue("Failed to ad elements properly",
					alist.get(i) == objArray[i - 50]);
		for (int i = 0; i >= 150 && (i < 200); i++)
			assertTrue("Failed to ad elements properly",
					alist.get(i) == objArray[i - 100]);
		ArrayList listWithNulls = new ArrayList();
		listWithNulls.add(null);
		listWithNulls.add(null);
		listWithNulls.add("yoink");
		listWithNulls.add("kazoo");
		listWithNulls.add(null);
		alist.addAll(100, listWithNulls);
		assertTrue("Incorrect size: " + alist.size(), alist.size() == 205);
		assertTrue("Item at slot 100 should be null", alist.get(100) == null);
		assertTrue("Item at slot 101 should be null", alist.get(101) == null);
		assertTrue("Item at slot 102 should be 'yoink'", alist.get(102).equals(
				"yoink"));
		assertTrue("Item at slot 103 should be 'kazoo'", alist.get(103).equals(
				"kazoo"));
		assertTrue("Item at slot 104 should be null", alist.get(104) == null);
		alist.addAll(205, listWithNulls);
		assertTrue("Incorrect size2: " + alist.size(), alist.size() == 210);
	}

	/**
	 * @tests java.util.ArrayList#addAll(java.util.Collection)
	 */
	public void test_addAllLjava_util_Collection() {
		// Test for method boolean
		// java.util.ArrayList.addAll(java.util.Collection)
		List l = new ArrayList();
		l.addAll(alist);
		for (int i = 0; i < alist.size(); i++)
			assertTrue("Failed to add elements properly", l.get(i).equals(
					alist.get(i)));
		alist.addAll(alist);
		assertTrue("Returned incorrect size after adding to existing list",
				alist.size() == 200);
		for (int i = 0; i < 100; i++) {
			assertTrue("Added to list in incorrect order", alist.get(i)
					.equals(l.get(i)));
			assertTrue("Failed to add to existing list", alist.get(i + 100)
					.equals(l.get(i)));
		}
		Set setWithNulls = new HashSet();
		setWithNulls.add(null);
		setWithNulls.add(null);
		setWithNulls.add("yoink");
		setWithNulls.add("kazoo");
		setWithNulls.add(null);
		alist.addAll(100, setWithNulls);
		Iterator i = setWithNulls.iterator();
		assertTrue("Item at slot 100 is wrong: " + alist.get(100), alist
				.get(100) == i.next());
		assertTrue("Item at slot 101 is wrong: " + alist.get(101), alist
				.get(101) == i.next());
		assertTrue("Item at slot 103 is wrong: " + alist.get(102), alist
				.get(102) == i.next());

	}

	/**
	 * @tests java.util.ArrayList#clear()
	 */
	public void test_clear() {
		// Test for method void java.util.ArrayList.clear()
		alist.clear();
		assertTrue("List did not clear", alist.size() == 0);
		alist.add(null);
		alist.add(null);
		alist.add(null);
		alist.add("bam");
		alist.clear();
		assertTrue("List with nulls did not clear", alist.size() == 0);
		/*
		 * for (int i = 0; i < alist.size(); i++) assertTrue("Failed to clear
		 * list", alist.get(i) == null);
		 */

	}

	/**
	 * @tests java.util.ArrayList#clone()
	 */
	public void test_clone() {
		// Test for method java.lang.Object java.util.ArrayList.clone()
		ArrayList x = (ArrayList) (((ArrayList) (alist)).clone());
		assertTrue("Cloned list was inequal to original", x.equals(alist));
		for (int i = 0; i < alist.size(); i++)
			assertTrue("Cloned list contains incorrect elements",
					alist.get(i) == x.get(i));

		alist.add(null);
		alist.add(25, null);
		x = (ArrayList) (((ArrayList) (alist)).clone());
		assertTrue("nulls test - Cloned list was inequal to original", x
				.equals(alist));
		for (int i = 0; i < alist.size(); i++)
			assertTrue("nulls test - Cloned list contains incorrect elements",
					alist.get(i) == x.get(i));

	}

	/**
	 * @tests java.util.ArrayList#contains(java.lang.Object)
	 */
	public void test_containsLjava_lang_Object() {
		// Test for method boolean
		// java.util.ArrayList.contains(java.lang.Object)
		assertTrue("Returned false for valid element", alist
				.contains(objArray[99]));
		assertTrue("Returned false for equal element", alist
				.contains(new Integer(8)));
		assertTrue("Returned true for invalid element", !alist
				.contains(new Object()));
		assertTrue("Returned true for null but should have returned false",
				!alist.contains(null));
		alist.add(null);
		assertTrue("Returned false for null but should have returned true",
				alist.contains(null));
	}

	/**
	 * @tests java.util.ArrayList#ensureCapacity(int)
	 */
	public void test_ensureCapacityI() {
		// Test for method void java.util.ArrayList.ensureCapacity(int)
		// TODO : There is no good way to test this as it only really impacts on
		// the private implementation.

		Object testObject = new Object();
		int capacity = 20;
		ArrayList al = new ArrayList(capacity);
		int i;
		for (i = 0; i < capacity / 2; i++) {
			al.add(i, new Object());
		}
		al.add(i, testObject);
		int location = al.indexOf(testObject);
		try {
			al.ensureCapacity(capacity);
			assertTrue("EnsureCapacity moved objects around in array1.",
					location == al.indexOf(testObject));
			al.remove(0);
			al.ensureCapacity(capacity);
			assertTrue("EnsureCapacity moved objects around in array2.",
					--location == al.indexOf(testObject));
			al.ensureCapacity(capacity + 2);
			assertTrue("EnsureCapacity did not change location.",
					location == al.indexOf(testObject));
		} catch (Exception e) {
			fail("Exception during test : " + e.getMessage());
		}
	}

	/**
	 * @tests java.util.ArrayList#get(int)
	 */
	public void test_getI() {
		// Test for method java.lang.Object java.util.ArrayList.get(int)
		assertTrue("Returned incorrect element", alist.get(22) == objArray[22]);
		try {
			alist.get(8765);
		} catch (IndexOutOfBoundsException e) {
			// Correct
			return;
		}
		fail("Failed to throw expected exception for index > size");
	}

	/**
	 * @tests java.util.ArrayList#indexOf(java.lang.Object)
	 */
	public void test_indexOfLjava_lang_Object() {
		// Test for method int java.util.ArrayList.indexOf(java.lang.Object)
		assertTrue("Returned incorrect index",
				alist.indexOf(objArray[87]) == 87);
		assertTrue("Returned index for invalid Object", alist
				.indexOf(new Object()) == -1);
		alist.add(25, null);
		alist.add(50, null);
		assertTrue("Wrong indexOf for null.  Wanted 25 got: "
				+ alist.indexOf(null), alist.indexOf(null) == 25);
	}

	/**
	 * @tests java.util.ArrayList#isEmpty()
	 */
	public void test_isEmpty() {
		// Test for method boolean java.util.ArrayList.isEmpty()
		assertTrue("isEmpty returned false for new list", new ArrayList()
				.isEmpty());
		assertTrue("Returned true for existing list with elements", !alist
				.isEmpty());
	}

	/**
	 * @tests java.util.ArrayList#lastIndexOf(java.lang.Object)
	 */
	public void test_lastIndexOfLjava_lang_Object() {
		// Test for method int java.util.ArrayList.lastIndexOf(java.lang.Object)
		alist.add(new Integer(99));
		assertTrue("Returned incorrect index",
				alist.lastIndexOf(objArray[99]) == 100);
		assertTrue("Returned index for invalid Object", alist
				.lastIndexOf(new Object()) == -1);
		alist.add(25, null);
		alist.add(50, null);
		assertTrue("Wrong lastIndexOf for null.  Wanted 50 got: "
				+ alist.lastIndexOf(null), alist.lastIndexOf(null) == 50);
	}

	/**
	 * @tests java.util.ArrayList#remove(int)
	 */
	public void test_removeI() {
		// Test for method java.lang.Object java.util.ArrayList.remove(int)
		alist.remove(10);
		assertTrue("Failed to remove element",
				alist.indexOf(objArray[10]) == -1);
		boolean exception = false;
		try {
			alist.remove(999);
		} catch (IndexOutOfBoundsException e) {
			// Correct
			exception = true;
		}
		assertTrue("Failed to throw exception when index out of range",
				exception);

		ArrayList myList = (ArrayList) (((ArrayList) (alist)).clone());
		alist.add(25, null);
		alist.add(50, null);
		alist.remove(50);
		alist.remove(25);
		assertTrue("Removing nulls did not work", alist.equals(myList));

		List list = new ArrayList(Arrays.asList(new String[] { "a", "b", "c",
				"d", "e", "f", "g" }));
		assertTrue("Removed wrong element 1", list.remove(0) == "a");
		assertTrue("Removed wrong element 2", list.remove(4) == "f");
		String[] result = new String[5];
		list.toArray(result);
		assertTrue("Removed wrong element 3", Arrays.equals(result,
				new String[] { "b", "c", "d", "e", "g" }));

		List l = new ArrayList(0);
		l.add(new Object());
		l.add(new Object());
		l.remove(0);
		l.remove(0);
		try {
			l.remove(-1);
			fail("-1 should cause exception");
		} catch (IndexOutOfBoundsException e) {
		}
		try {
			l.remove(0);
			fail("0 should case exception");
		} catch (IndexOutOfBoundsException e) {
		}
	}

	/**
	 * @tests java.util.ArrayList#set(int, java.lang.Object)
	 */
	public void test_setILjava_lang_Object() {
		// Test for method java.lang.Object java.util.ArrayList.set(int,
		// java.lang.Object)
		Object obj;
		alist.set(65, obj = new Object());
		assertTrue("Failed to set object", alist.get(65) == obj);
		alist.set(50, null);
		assertTrue("Setting to null did not work", alist.get(50) == null);
		assertTrue("Setting increased the list's size to: " + alist.size(),
				alist.size() == 100);
	}

	/**
	 * @tests java.util.ArrayList#size()
	 */
	public void test_size() {
		// Test for method int java.util.ArrayList.size()
		assertTrue("Returned incorrect size for exiting list",
				alist.size() == 100);
		assertTrue("Returned incorrect size for new list", new ArrayList()
				.size() == 0);
	}

	/**
	 * @tests java.util.ArrayList#toArray()
	 */
	public void test_toArray() {
		// Test for method java.lang.Object [] java.util.ArrayList.toArray()
		alist.set(25, null);
		alist.set(75, null);
		Object[] obj = alist.toArray();
		assertEquals("Returned array of incorrect size", objArray.length,
				obj.length);

		for (int i = 0; i < obj.length; i++) {
			if ((i == 25) || (i == 75))
				assertTrue("Should be null at: " + i + " but instead got: "
						+ obj[i], obj[i] == null);
			else
				assertTrue("Returned incorrect array: " + i,
						obj[i] == objArray[i]);
		}

	}

	/**
	 * @tests java.util.ArrayList#toArray(java.lang.Object[])
	 */
	public void test_toArray$Ljava_lang_Object() {
		// Test for method java.lang.Object []
		// java.util.ArrayList.toArray(java.lang.Object [])
		alist.set(25, null);
		alist.set(75, null);
		Integer[] argArray = new Integer[100];
		Object[] retArray;
		retArray = alist.toArray(argArray);
		assertTrue("Returned different array than passed", retArray == argArray);
		argArray = new Integer[1000];
		retArray = alist.toArray(argArray);
		assertTrue("Failed to set first extra element to null", argArray[alist
				.size()] == null);
		for (int i = 0; i < 100; i++) {
			if ((i == 25) || (i == 75))
				assertTrue("Should be null: " + i, retArray[i] == null);
			else
				assertTrue("Returned incorrect array: " + i,
						retArray[i] == objArray[i]);
		}
	}

	/**
	 * @tests java.util.ArrayList#trimToSize()
	 */
	public void test_trimToSize() {
		// Test for method void java.util.ArrayList.trimToSize()
		for (int i = 99; i > 24; i--)
			alist.remove(i);
		((ArrayList) alist).trimToSize();
		assertTrue("Returned incorrect size after trim", alist.size() == 25);
		for (int i = 0; i < alist.size(); i++)
			assertTrue("Trimmed list contained incorrect elements", alist
					.get(i) == objArray[i]);
	}

	/**
	 * Sets up the fixture, for example, open a network connection. This method
	 * is called before a test is executed.
	 */
	protected void setUp() {
		alist = new ArrayList();
		for (int i = 0; i < objArray.length; i++)
			alist.add(objArray[i]);
	}

	/**
	 * Tears down the fixture, for example, close a network connection. This
	 * method is called after a test is executed.
	 */
	protected void tearDown() {
	}
}
