/* Copyright 1998, 2005 The Apache Software Foundation or its licensors, as applicable
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package tests.api.java.util;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

import tests.support.resource.Support_Resources;

public class ResourceBundleTest extends junit.framework.TestCase {

	/**
	 * @tests java.util.ResourceBundle#getBundle(java.lang.String,
	 *        java.util.Locale)
	 */
	public void test_getBundleLjava_lang_StringLjava_util_Locale() {
		ResourceBundle bundle;
		String name = "tests.support.Support_TestResource";
		Locale defLocale = Locale.getDefault();

		Locale.setDefault(new Locale("en", "US"));
		bundle = ResourceBundle.getBundle(name, new Locale("fr", "FR", "VAR"));
		assertTrue("Wrong bundle fr_FR_VAR", bundle.getString("parent4")
				.equals("frFRVARValue4"));
		bundle = ResourceBundle.getBundle(name, new Locale("fr", "FR", "v1"));
		assertTrue("Wrong bundle fr_FR_v1", bundle.getString("parent4").equals(
				"frFRValue4"));
		bundle = ResourceBundle.getBundle(name, new Locale("fr", "US", "VAR"));
		assertTrue("Wrong bundle fr_US_var", bundle.getString("parent4")
				.equals("frValue4"));
		bundle = ResourceBundle.getBundle(name, new Locale("de", "FR", "VAR"));
		assertTrue("Wrong bundle de_FR_var", bundle.getString("parent4")
				.equals("enUSValue4"));

		Locale.setDefault(new Locale("fr", "FR", "VAR"));
		bundle = ResourceBundle.getBundle(name, new Locale("de", "FR", "v1"));
		assertTrue("Wrong bundle de_FR_var 2", bundle.getString("parent4")
				.equals("frFRVARValue4"));

		Locale.setDefault(new Locale("de", "US"));
		bundle = ResourceBundle.getBundle(name, new Locale("de", "FR", "var"));
		assertTrue("Wrong bundle de_FR_var 2", bundle.getString("parent4")
				.equals("parentValue4"));

		// Test with a security manager
		Locale.setDefault(new Locale("en", "US"));
		System.setSecurityManager(new SecurityManager());
		try {
			bundle = ResourceBundle.getBundle(name, new Locale("fr", "FR",
					"VAR"));
			assertTrue("Security: Wrong bundle fr_FR_VAR", bundle.getString(
					"parent4").equals("frFRVARValue4"));
			bundle = ResourceBundle.getBundle(name,
					new Locale("fr", "FR", "v1"));
			assertTrue("Security: Wrong bundle fr_FR_v1", bundle.getString(
					"parent4").equals("frFRValue4"));
			bundle = ResourceBundle.getBundle(name, new Locale("fr", "US",
					"VAR"));
			assertTrue("Security: Wrong bundle fr_US_var", bundle.getString(
					"parent4").equals("frValue4"));
			bundle = ResourceBundle.getBundle(name, new Locale("de", "FR",
					"VAR"));
			assertTrue("Security: Wrong bundle de_FR_var: "
					+ bundle.getString("parent4"), bundle.getString("parent4")
					.equals("enUSValue4"));
		} finally {
			System.setSecurityManager(null);
		}

		Locale.setDefault(defLocale);
	}

	/**
	 * @tests java.util.ResourceBundle#getBundle(java.lang.String,
	 *        java.util.Locale, java.lang.ClassLoader)
	 */
	public void test_getBundleLjava_lang_StringLjava_util_LocaleLjava_lang_ClassLoader() {
		String classPath = System.getProperty("java.class.path");
		StringTokenizer tok = new StringTokenizer(classPath, File.pathSeparator);
		Vector urlVec = new Vector();
		String resPackage = Support_Resources.RESOURCE_PACKAGE;
		try {
			while (tok.hasMoreTokens()) {
				String path = (String) tok.nextToken();
				String url;
				if (new File(path).isDirectory())
					url = "file:" + path + resPackage + "subfolder/";
				else
					url = "jar:file:" + path + "!" + resPackage + "subfolder/";
				urlVec.addElement(new URL(url));
			}
		} catch (MalformedURLException e) {
		}
		URL[] urls = new URL[urlVec.size()];
		for (int i = 0; i < urlVec.size(); i++)
			urls[i] = (URL) urlVec.elementAt(i);
		URLClassLoader loader = new URLClassLoader(urls, null);

		String name = Support_Resources.RESOURCE_PACKAGE_NAME
				+ ".hyts_resource";
		ResourceBundle bundle = ResourceBundle.getBundle(name, Locale
				.getDefault());
		bundle = ResourceBundle.getBundle(name, Locale.getDefault(), loader);
		assertTrue("Wrong cached value", bundle.getString("property").equals(
				"resource"));
	}

	/**
	 * @tests java.util.ResourceBundle#getString(java.lang.String)
	 */
	public void test_getStringLjava_lang_String() {
		ResourceBundle bundle;
		String name = "tests.support.Support_TestResource";
		Locale.setDefault(new Locale("en", "US"));
		bundle = ResourceBundle.getBundle(name, new Locale("fr", "FR", "VAR"));
		assertTrue("Wrong value parent4", bundle.getString("parent4").equals(
				"frFRVARValue4"));
		assertTrue("Wrong value parent3", bundle.getString("parent3").equals(
				"frFRValue3"));
		assertTrue("Wrong value parent2", bundle.getString("parent2").equals(
				"frValue2"));
		assertTrue("Wrong value parent1", bundle.getString("parent1").equals(
				"parentValue1"));
		assertTrue("Wrong value child3", bundle.getString("child3").equals(
				"frFRVARChildValue3"));
		assertTrue("Wrong value child2", bundle.getString("child2").equals(
				"frFRVARChildValue2"));
		assertTrue("Wrong value child1", bundle.getString("child1").equals(
				"frFRVARChildValue1"));
	}

	protected void setUp() {
	}

	protected void tearDown() {
	}
}
